//go:build !disable_format_aac && !disable_codec_libfdk_aac && cgo

package aac

import (
	"git.gammaspectra.live/S.O.N.G/Kirika/audio"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio/format/flac"
	"git.gammaspectra.live/S.O.N.G/Kirika/test"
	"git.gammaspectra.live/S.O.N.G/go-fdkaac/fdkaac"
	"io"
	"os"
	"testing"
)

func TestEncodeAAC(t *testing.T) {
	t.Parallel()
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	target, err := os.CreateTemp("/tmp", "encode_test_*.aac")
	if err != nil {
		t.Error(err)
		return
	}

	defer func() {
		name := target.Name()
		target.Close()
		os.Remove(name)
	}()

	options := make(map[string]interface{})
	options["bitrate"] = "256k"

	err = NewFormat().Encode(source, target, options)
	if err != nil {
		t.Error(err)
		return
	}
}

func TestDecodeAAC(t *testing.T) {
	t.Parallel()
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	target, err := os.CreateTemp("/tmp", "decode_test_*.aac")
	if err != nil {
		t.Error(err)
		return
	}

	defer func() {
		name := target.Name()
		target.Close()
		os.Remove(name)
	}()

	options := make(map[string]interface{})
	options["bitrate"] = "256k"

	err = NewFormat().Encode(source, target, options)
	if err != nil {
		t.Error(err)
		return
	}

	target.Seek(0, io.SeekStart)

	source, err = NewFormat().Open(target)
	if err != nil {
		t.Error(err)
		return
	}

	//Decode
	audio.NewNullSink().Process(source)
}

/*
func TestDecodeAACMP4(t *testing.T) {
	t.Parallel()
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	target, err := os.Open(test.TestSingleSampleAACMP4)
	if err != nil {
		t.Error(err)
		return
	}
	defer target.Close()

	source, err = NewFormat().Open(target)
	if err != nil {
		t.Error(err)
		return
	}

	audio.NewNullSink().Process(source)
}
*/

func TestEncodeAACMP4(t *testing.T) {
	t.Parallel()
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	target, err := os.CreateTemp("/tmp", "encode_test_*.m4a")
	if err != nil {
		t.Error(err)
		return
	}

	defer func() {
		name := target.Name()
		target.Close()
		os.Remove(name)
	}()

	options := make(map[string]interface{})
	options["bitrate"] = "256k"
	options["format"] = "mp4"

	err = NewFormat().Encode(source, target, options)
	if err != nil {
		t.Error(err)
		return
	}
}

func TestEncodeAACHEv2(t *testing.T) {
	t.Parallel()
	if !fdkaac.EncoderCapabilities().PS {
		return
	}
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	target, err := os.CreateTemp("/tmp", "encode_test_*.aac")
	if err != nil {
		t.Error(err)
		return
	}

	defer func() {
		name := target.Name()
		target.Close()
		os.Remove(name)
	}()

	options := make(map[string]interface{})
	options["bitrate"] = 32
	options["mode"] = "hev2"

	err = NewFormat().Encode(source, target, options)
	if err != nil {
		t.Error(err)
		return
	}
}
