//go:build !disable_format_aac && cgo

package packetizer

import (
	"git.gammaspectra.live/S.O.N.G/Kirika/audio/format/aac"
	"git.gammaspectra.live/S.O.N.G/Kirika/audio/format/flac"
	"git.gammaspectra.live/S.O.N.G/Kirika/test"
	"io"
	"os"
	"testing"
)

func TestPacketizeADTS(t *testing.T) {
	t.Parallel()
	fp, err := os.Open(test.SingleSample24)
	if err != nil {
		t.Error(err)
		return
	}
	defer fp.Close()
	source, err := flac.NewFormat().Open(fp)
	if err != nil {
		t.Error(err)
		return
	}

	reader, writer := io.Pipe()

	go func() {
		defer writer.Close()
		err = aac.NewFormat().Encode(source, writer, nil)
		if err != nil {
			t.Error(err)
			return
		}
	}()

	pipePacketizer := NewAdtsPacketizer(reader)
	packetCount := 0
	packetBytes := 0
	for {
		packet := pipePacketizer.GetPacket()
		if packet == nil {
			break
		}
		packetCount++
		packetBytes += len(packet.GetData())
	}

	//Double checks to match vo-aacenc
	if packetCount != 16920 && packetCount != 16917 {
		t.Errorf("Wrong Packet Count %d != %d", packetCount, 16920)
	}
	if packetBytes < 6000000 {
		t.Errorf("Wrong Packet Bytes %d < %d", packetBytes, 6000000)
	}
}
